#ifndef _INC_ALMARDUINO_H
#define _INC_ALMARDUINO_H

//! \file AlMarDuino.h
//! \brief Archivo de cabecera para utilizar la librería de E/S estilo Arduino con el simulador de Sistemas Físicos.
//!
//! Para compilación con QtCreator, añadir lo siguiente al .pro (cambiando el contenido de SIMUL_LIBRARY_PATH por el directorio donde se haya descomprimido la librería).<br>
//! Archivo .pro:<br>
//! \code{.unparsed}
//! SIMUL_LIBRARY_PATH = C:/User/.../Client/Library
//! INCLUDEPATH += $${SIMUL_LIBRARY_PATH}/include
//! contains(QT_ARCH, i386) {
//!     message("32-bit")
//!     LIBS += -L$${SIMUL_LIBRARY_PATH}/mingw32 -lUserLibAlMarDuino
//! } else {
//!     message("64-bit")
//!     LIBS += -L$${SIMUL_LIBRARY_PATH}/mingw64 -lUserLibAlMarDuino
//! }
//! # Si se desea utilizar curses:
//! LIBS += -lpdcurses
//! # Si se desea utilizar windows sockets:
//! LIBS += -lws2_32
//! \endcode
//! <br>
//! Ejemplo de uso<br>
//! test.c:<br>
//! \code{.c}
//! #include <stdio.h>
//! #include "AlMarDuino.h"
//! #include "AlMarDuino_Servo.h"
//! // using namespace AlMarDuino;  // Uncomment to avoid need of explicit namespace
//! //                                 declaration for every AlMarDuino function or class
//!
//! AlMarDuino_MAIN          // Integrates main() and command line arguments evaluation
//! double angles_servo[]={90,135,60,45};
//! AlMarDuino::Servo servos[4];

//! void setup()
//! {
//!    for (int j=0;j<4;j++) {
//!        servos[j].attach(j);
//!        servos[j].write(angles_servo[j]);
//!    }
//! }
//!
//! void loop()
//! {
//!    int iServo;
//!    ... wait for text, or other condition ...
//!    if (condition is met) {
//!       angles_servo[iServo]=new value 0 to 180;
//!       servos[iServo].write(angles_servo[iServo]);
//!    }
//! }
//! \endcode

//! Función setup() a ser definida por el programador
void setup();

//! Función loop() a ser definida por el programador
void loop();

//! Arguments can be indicated in the command line (if not, they are requested to the user):<br>
//!   "--appname=AlMarDrawingRobot"<br>
//!   "--username=alumno"<br>
//!   "--passwd=the_encrypted_passwd"<br>
//!   "--ip=127.0.0.1"<br>
//!   "--port=8080"
#define AlMarDuino_MAIN int main(int argc,const char* argv[ ]) { \
    AlMarDuino::Internal::Setup(argc,argv); \
    setup(); \
    while (1) { \
        loop(); \
    } \
    return 0; \
}

#include <stdint.h>


namespace AlMarDuino {
//! Delays execution <a href="https://www.arduino.cc/reference/es/language/functions/time/delay/">see Arduino equivalent function</a>
//! \param time_ms: time to delay in ms
void delay(int time_ms);
bool digitalRead(int pin);
void digitalWrite(int pin);
uint32_t millis();
uint32_t micros();

namespace Internal {
int Setup(int argc,const char* argv[]);
} // namespace Internal

} //namespace AlMarDuino

#endif // _INC_ALMARDUINO_H
