function sock=AlMarConnectWs(appName,user,pwd,ip,port)
% sock=AlMarConnectWs(appName,user,pwd,ip,port)
% values for the args can be obtained from the web interface activating "Show client Info" checkbox
% appName (string): name of the simulator application to connect with
% user (string): username
% pwd (string): encrypted password
% ip (string): ip of the server (typically '127.0.0.1' in local execution)
% port (number): port number of the server
%
% return: object of type AlMarWebSocket

    if ~isstr(appName)
        appName=convertStringsToChars(appName);
    end
    if ~isstr(user)
        user=convertStringsToChars(user);
    end
    if ~isstr(pwd)
        pwd=convertStringsToChars(pwd);
    end
    if ~isstr(ip)
        ip=convertStringsToChars(ip);
    end
    % connect to base url

    sock=tcpclient(ip,port);
    sock.configureTerminator('CR/LF');
    crlf=sprintf('\r\n');
    a=AlMarWebSocket.WriteReadAll(sock,sprintf('GET: %s HTTP 1.1\r\nHost:"%s:%d"\r\n','/',ip,port));

    ans=AlMarWebSocket.GetSubstring(a,'HTTP/1.1',' ',crlf);
    if isempty(ans)
        return;
    end

    % the base uri requests Digest autenthication: prepare answer headers and send
    auth_req_str=AlMarWebSocket.GetSubstring(a,'WWW-Authenticate: Digest',' ',sprintf('\r\n'));
    if isempty(auth_req_str)
        sock.close();
        return;
    end
    auth_req.realm=AlMarWebSocket.GetSubstring(auth_req_str,'realm','="','"');
    auth_req.uri=AlMarWebSocket.GetSubstring(auth_req_str,'uri','="','"');
    auth_req.nonce=AlMarWebSocket.GetSubstring(auth_req_str,'nonce','="','"');
    auth_req.algorithm=AlMarWebSocket.GetSubstring(auth_req_str,'algorithm','="','"');

    cnonce='302707';
    nc='00001234';
    qop='auth';
    ha1=pwd;
    hash256_class   = System.Security.Cryptography.SHA256Managed.Create();
    ha2_hashed  = uint8(hash256_class.ComputeHash(uint8(['GET:',auth_req.uri])));
    ha2=sprintf('%02x',ha2_hashed);
    response_raw=[ha1,':',auth_req.nonce,':',nc,':',cnonce,':',qop,':',ha2];
    response_hashed  = uint8(hash256_class.ComputeHash(uint8(response_raw)));
    response=sprintf('%02x',response_hashed);

    auth_ans=sprintf('Digest username="%s",realm="%s",nonce="%s",algorithm="%s",uri="%s",qop="%s",cnonce="%s",nc="%s",response="%s"', ...
                     user,auth_req.realm,auth_req.nonce,auth_req.algorithm,auth_req.uri,qop,cnonce,nc,response);
    sock=tcpclient(ip,port);
    a=AlMarWebSocket.WriteReadAll(sock,sprintf('GET: %s HTTP 1.1\r\nHost:"%s:%d"\r\nAuthorization: %s\r\n','/',ip,port,auth_ans));

    % Read authentication answer
    ans=AlMarWebSocket.GetSubstring(a,'HTTP/1.1',' ',crlf);
    if isempty(ans)
        clear sock;
        sock=[];
        return;
    end
    if length(strfind(ans,'303 See Other'))==0
        clear sock;
        sock=[];
        return;
    end
    
    % Promote to websocket and return
    sock=AlMarWebSocket(sock,['/Apps/' appName '/wss/' user '/&type=userLib'],auth_ans);
end


