classdef AlMarServoControl
    % AlMarServoControl: Class that manages the comm with an AlMarWebSocket to output servo values

    properties
        servos
        snd
        ws
    end

    methods
        function obj = AlMarServoControl(ws_conn,i_servos,i_pwmOn_us,i_pwmT_us)
            %AlMarServoControl Constructor: inits PWM channels (Tpwm and Ton) 
            %   ws_conn (AlMarWebSocket object): instance of the websocket connected 
            %   i_servos (array of int): indexes of the servos to manage (values 0 to 3)
            %   i_pwmOn_us (array of int): array of same size of i_servos with the initial PWM values (0 to i_pwmT_us)
            %   i_pwmT_us (int): period for the PWM (defaults to 20000 if not supplied)

            obj.ws=ws_conn;
            obj.servos=i_servos;
            obj.snd=struct('computer_outputs', []);
            if nargin<4
                i_pwmT_us=20000;
            end
            obj.snd.computer_outputs=struct('name', 'PWM', 'iotype', sprintf('pwm%d',i_servos(1)), 't_on', i_pwmOn_us(1), 't_total', i_pwmT_us);
            for ind=2:length(i_servos)
                obj.snd.computer_outputs(ind)=struct('name', 'PWM', 'iotype', sprintf('pwm%d',i_servos(ind)), 't_on', i_pwmOn_us(ind), 't_total', i_pwmT_us);
            end
            obj.ws.Send(obj.snd);
        end

        function SetPwm(obj,i_Servos,i_pwmOn_us)
            %SetPwm: changes pwm on time for specified servos
            %   ws_conn (AlMarWebSocket object): instance of the websocket connected 
            %   i_servos (array of int): indexes of the servos to change (values 0 to 3)
            %   i_pwmOn_us (array of int): array of same size of i_servos with the new PWM values (0 to i_pwmT_us)

            for ind=1:length(i_Servos)
                found=find(obj.servos==i_Servos(ind));
                if ~isempty(found)
                    obj.snd.computer_outputs(found).t_on=i_pwmOn_us(ind);
                end
            end
            obj.ws.Send(obj.snd);
        end
    end
end