classdef AlMarWebSocket
    % AlMarWebSocket: Class that manages the upgrade to websocket and then allows send/receive tasks (synchronous) 
    %                 Send/receive can work with jSon text or with Matlab structs that are converted to/from jSon text

    properties
        sock;
        init=false;
    end

    methods
        function obj = AlMarWebSocket(sock,uri,auth_headers)
            %AlMarWebSocket Constructor: switches protocol to websocket
            %   sock (tcpclient object): instance of tcpclient class already connected with the server
            %   uri (string): uri where the server answers to the websocket
            %   auth_headers(string): authentication headers to add to the communication

            obj.sock = sock;
            obj.sock.configureTerminator('CR/LF');
            obj.sock.ErrorOccurredFcn= @AlMarWebSocket.OnError;

            s_ws_key='x3JJHMbDL1EzLkh9GBhXDw=='; % TODO: create 
		    s_ws_protocol='chat';

            a=obj.WriteRead([sprintf('GET: %s HTTP 1.1\r\nHost:"%s:%d"\r\n',uri,obj.sock.Address,obj.sock.Port), ...
                                sprintf('Upgrade:"websocket"\r\nConnection:"upgrade"\r\n'), ...
                                sprintf('Sec-WebSocket-Key:"%s"\r\n',s_ws_key), ...
                                sprintf('Sec-WebSocket-Protocol:"%s"\r\n',s_ws_protocol), ...
                                sprintf('Sec-WebSocket-Version:%d\r\n',13), ...
                                sprintf('Origin:"matlabClient"\r\n'), ...
                                sprintf('Authorization: %s\r\n',auth_headers), ...
                               ]);
            ans=AlMarWebSocket.GetSubstring(a,'HTTP/1.1',' ',sprintf('\r\n'));
            if strcmp(ans,'101 Switching Protocols')
                obj.init=true;
            end
            return;
        end

        function n=Send(obj,value)
            % Send: Send data over wss 
            %   value (string or struct): data to send according with the Simulator protocol
            %                             (jSon string, or struct that is automatically converted to jSon)
            %   return: number of bytes written
            if ~isstr(value)
                len=length(value.computer_outputs); 
                value=jsonencode(value);
                if len==1 % Matlab encodes arrays of length 1 as a single value, not as an array
                    index1=strfind(value,'{"name"');
                    index2=length(value)-1;
                    value=[value(1:index1-1),'[',value(index1:index2),']',value(index2+1:end)];
                end
            end
            nb=uint16(length(value));
            v=[];
            if nb<126
                v=uint8([0x81,bitor(0x80,nb),0,0,0,0]);
            else
                v=uint8([uint16(0x81),bitor(126,uint16(0x80)),bitand(bitshift(nb,-8),uint16(0xFF)),bitand(nb,uint16(0xFF)),uint16([0,0,0,0])]);
            end
            obj.sock.write([v,value]);
            n=obj.sock.NumBytesAvailable;
        end
    end

    methods(Access=private)
        function ans=WriteRead(obj,snd)
            ans=AlMarWebSocket.WriteReadAll(obj.sock,snd);
        end

        function ans=Receive(obj)
            %WriteRead Receive data over wss 
            %   snd (string or struct): data to send according with the Simulator protocol
            %                           (jSon string, or struct that is automatically converted to jSon)
            ans='';
            if ~obj.init
                return;
            end

            ws_head=obj.sock.read(2);
            fin=bitshift(bitand(ws_head(1),0x80),-7);
            opcode=bitand(ws_head(1),0x0F);
            nb=bitand(ws_head(2),0x7F);
            mask=bitshift(bitand(ws_head(2),0x80),-7);
            istart=2;
            if opcode==1
                if mask~=0
                    istart=istart+4;
                end
                if nb==126
                    istart=istart+2;
                    ws_len=obj.sock.read(2);
                    nb=bitshift(int16(ws_len(1)),8)+int16(ws_len(2));
                end
                ans=jsondecode(char(obj.sock.read(nb)));
            end
        end

    end

    methods(Static)
        function value=GetSubstring(str,field,del1,del2)
            index=strfind(str,[field,del1]);
            if length(index)==0
                value='';
                return;
            end
            str=str(index(1)+length(field)+length(del1):end);
            index=strfind(str,del2);
            if length(index)==0
                value='';
                return;
            end
            value=str(1:index(1)-1);
        end
        
        function a=WriteReadAll(sock,snd)
            a=sock.writeread(snd);
            while 1
                more=sock.read();
                if isempty(more)
                   break;
                end
                a=[a,sprintf('\r\n'),more];
            end
        end

        function OnError()
            disp('Error in socket. Please reconnect\n');
        end

    end
end