# https://websocket-client.readthedocs.io/en/latest/examples.html#connection-options

# import websocket (Cannot use Python internal websocket, does not accept auth headers before the negotiation protocol)
import http.client
import hashlib
import json

class AlMarWebSocket:
	def __init__(self,host,path,auth_headers):
		self.sock={}
	
		ws_cli=http.client.HTTPConnection(host)
		s_ws_key='x3JJHMbDL1EzLkh9GBhXDw=='	# TODO: create 
		s_ws_protocol='chat'
		ws_headers={'Host':host, \
						'Upgrade':'websocket', \
						'Connection':'upgrade', \
						'Sec-WebSocket-Key' :  s_ws_key , \
						'Sec-WebSocket-Protocol' : s_ws_protocol, \
						'Sec-WebSocket-Version' : 13, \
						'Origin' : 'pythonClient' \
						}
		ws_headers.update(auth_headers)
		ws_cli.request('GET',path,headers=ws_headers)
		self.ans=ws_cli.getresponse()
		ans=self.ans
		ans.read()
		
		if ans.status==101 and ans.reason=='Switching Protocols':
			h=ans.headers
			if h.__getitem__('Connection')=='Upgrade' and h.__getitem__('Upgrade')=='websocket':
				self.sock=ws_cli.sock
				del ws_cli
				s_ws_accept=h.__getitem__('Sec-WebSocket-Accept')
				s_ws_protocol=h.__getitem__('Sec-WebSocket-Protocol')
				hash=s_ws_key+'258EAFA5-E914-47DA-95CA-C5AB0DC85B11'
				# TODO sha1(hash) --> base64 --> check with s_ws_accept
				print('Connected')
			else:
				print('Cannot connect with server')
		else:
			print('Cannot connect with server')
				
	def Send(self,msg) :		
		# Byte 0: flags (fin) , opcode (text frame)
		# Byte 1: mask (client) , payload length (if 126, means extended payload length is needed)
		# Bytes 2,3: extended payload length (MSB, LSB), if needed (payload length==126)
		# Bytes 4,5,6,7: masking key (zero means no masking) (MSB to LSB)
		if type(msg).__name__ is str:
			txt_msg=msg
		else:	# object is supposed, convert to json
			txt_msg=json.dumps(msg)
			
		nb=len(txt_msg)
		v=None
		if nb<126:
			v=[0b10000001 , 0b10000000 | nb, \
				0,0,0,0 ]
		else :
			v=[0b10000001 , 0b11111110, \
				(nb>>8) & 0xFF, nb & 0xFF, \
				0,0,0,0 ]
			
		ba=bytearray(v)+txt_msg.encode('utf-8')
		return self.sock.send(ba)

	def Recv(self,type='obj') :
		ba=self.sock.recv(4096)
		fin=(ba[0]&0x80) >> 7 
		opcode=ba[0]&0xF
		nb=ba[1] & 0x7F
		mask=ba[1] & 0x80 >> 7
		istart=2
		if opcode==1:	# text frame
			if mask!=0:
				istart=istart+4
			if nb==126:
				istart=istart+2
				nb=(ba[2]<<8) | ba[3]
		
			str=ba[istart:istart+nb].decode('utf-8')
			if type=='str':
				return str
			else:
				return json.loads(str)
		return None
		
def AlMarDigestAuthHeaderParse(header_str):
	ret={}
	i_field=0
	
	while True:
		i_equal=header_str.find('=',i_field)
		if i_equal<0:
			break
		field=header_str[i_field:i_equal]
		i_quote1=header_str.find('"',i_equal+1)
		i_quote2=header_str.find('"',i_quote1+1)
		value=header_str[i_quote1+1:i_quote2]
		ret[field]=value
		i_field=header_str.find(',',i_quote2+1)
		if i_field<0:
			break
		i_field=i_field+1
	return ret

def AlMarConnectWs(app,user,pwd,ip,port):
	uri=ip+':'+str(port)
	http_cli=http.client.HTTPConnection(uri)
#	http_cli.set_debuglevel(2)
	http_cli.request('GET','/')
	ans=http_cli.getresponse()
	ans.read()
	
	auth_req=ans.headers['WWW-Authenticate']
	
	auth_headers=None
	if auth_req!=None :
		if auth_req.startswith('Digest '):
			fields=AlMarDigestAuthHeaderParse(auth_req[7:])

#			print(fields)
			cnonce='MyCnonce'
			nc='00000123'
			qop='auth'
			
			auth_ans='Digest '
			auth_ans+='username="'+user+'",'
			auth_ans+='realm="'+fields['realm']+'",'
			auth_ans+='nonce="'+fields['nonce']+'",'
			auth_ans+='algorithm="'+fields['algorithm']+'",'
			auth_ans+='uri="'+fields['uri']+'",'
			auth_ans+='qop="'+qop+'",'
			auth_ans+='cnonce="'+cnonce+'",'
			auth_ans+='nc="'+nc+'",'
			
			ha1=pwd
			ha2=hashlib.sha256(('GET:'+fields['uri']).encode('utf-8')).hexdigest()
			response_raw=ha1+':'+fields['nonce']+':'+nc+':'+cnonce+':'+qop+':'+ha2
			response=hashlib.sha256(response_raw.encode('utf-8')).hexdigest()
			auth_ans+='response="'+response+'"'
			auth_headers={'Authorization':auth_ans,'Host':uri}
			http_cli.request('GET','/',headers=auth_headers)
			
			ans=http_cli.getresponse()	
			ans.read()
			if ans.code!=303:
				return

	del http_cli
	
	ws=AlMarWebSocket(uri,'/Apps/'+app+'/wss/'+user+'/&type=userLib',auth_headers)
	
	return ws
	
class AlMarServoControl:
	def __init__(self,ws,i_servos,i_pwmOn_us,i_pwmT_us=20000):
		self.ws=ws
		self.servos=i_servos
		self.snd={}
		self.snd['computer_outputs']=[]
		for i,i_servo in enumerate(self.servos):
			self.snd['computer_outputs'].append({'name':'PWM','iotype':'pwm'+str(i_servo),'t_on':i_pwmOn_us[i],'t_total':i_pwmT_us })
		self.ws.Send(self.snd)
		
	def SetPwm(self,i_servos,t_pwm_us):	
		if type(i_servos).__name__=='int':
			found=self.servos.index(i_servos)
			self.snd['computer_outputs'][found]['t_on']=t_pwm_us
		else:	# assumed array
			for i,i_servo in enumerate(i_servos):
				found=self.servos.index(i_servo)
				self.snd['computer_outputs'][found]['t_on']=t_pwm_us[i]
				
		self.ws.Send(self.snd)
		